@@ -23,9 +23,9 @@ newSOCKnode <- function(machine = "localhost", ...,
     homogeneous <- getClusterOption("homogeneous", options)
     if (getClusterOption("useRscript", options)) {
         if (homogeneous) {
-            rscript <- getClusterOption("rscript", options)
+            rscript <- shQuoteIfNeeded(getClusterOption("rscript", options))
             snowlib <- getClusterOption("snowlib", options)
-            script <- file.path(snowlib, "snow", "RSOCKnode.R")
+            script <- shQuoteIfNeeded(file.path(snowlib, "snow", "RSOCKnode.R"))
             env <- paste("MASTER=", master,
                          " PORT=", port,
                          " OUT=", outfile,
@@ -43,9 +43,9 @@ newSOCKnode <- function(machine = "localhost", ...,
     else {
         if (homogeneous) {
             scriptdir <- getClusterOption("scriptdir", options)
-            script <- file.path(scriptdir, "RSOCKnode.sh")
+            script <- shQuoteIfNeeded(file.path(scriptdir, "RSOCKnode.sh"))
             rlibs <- paste(getClusterOption("rlibs", options), collapse = ":")
-            rprog <- getClusterOption("rprog", options)
+            rprog <- shQuoteIfNeeded(getClusterOption("rprog", options))
             env <- paste("MASTER=", master,
                          " PORT=", port,
                          " OUT=", outfile,
